﻿/// <reference path="http://ajax.googleapis.com/ajax/libs/jquery/1.7.0/jquery.min.js"/>

/**
 * Creates an instance of a server media library
 *
 * @constructor
 * @author Suriel Bendahan
 * @version 1.0
 * @param userId the unique id the library is managed for
 * @param baseUrl The service base url calls are made to
 */
MediaLibrary = function(userId, baseUrl)
{
    /** 
     * Constructs a query string from a map
     * @author Suriel Bendahan
     * @private
     * @param args {Object} a map of simple values
     */
    var buildQString = function(args)
    {
    
        args = args || {};
        return '?' +
        _.map(args, function(v, k)
        {
            return encodeURIComponent(k) + "=" + encodeURIComponent(v);
        }).join('&');
        
    };
    
    /**
     * Creates an instance of an itemManager service wrapper
     *
     * @constructor
     * @param type media or playlist - the endpoint the manager is to work against
     */
    var itemManager = function(type)
    {
    
        /**
         * Retrieves a specific item from the server
         *
         * @author Suriel Bendahan
         * @param id the item id to retrieve
         * @param cbOk callback to call on success with the result
         * @param cbError callback to call with an error object if the operation failed
         */
        this.get = function(id, cbOk, cbError)
        {
            AjaxRequest.makeRequest(
            {
                contentType: 'application/json',
                dataType: 'json',
                type: 'GET',
                url: [baseUrl, userId, type, id].join('/'),
                success: function(d)
                {
                    if (d.success) 
                    {
                        cbOk(d.result);
                    }
                    else 
                        if (cbError) 
                            cbError(d);
                },
                error: function(x, e)
                {
                    if (cbError) 
                        cbError(
                        {
                            success: false,
                            error: e
                        });
                }
            });
        };
        
        /**
         * Adds a new item to the server
         *
         * @author Suriel Bendahan
         * @param item the item to add to the server
         * @param cbOk callback to call on success with the result
         * @param cbError callback to call with an error object if the operation failed
         */
        this.add = function(item, cbOk, cbError)
        {
            AjaxRequest.makeRequest(
            {
                contentType: 'application/json',
                data: item,
                dataType: 'json',
                type: 'PUT',
                url: [baseUrl, userId, type, ''].join('/'),
                success: function(d)
                {
                    if (d.success) 
                    {
                        item.id = d.result;
                        cbOk(item);
                    }
                    else 
                        if (cbError) 
                            cbError(d);
                },
                error: function(x, e)
                {
                    if (cbError) 
                        cbError(
                        {
                            success: false,
                            error: e
                        });
                }
            });
        };
        /**
         * Removes and item from the server
         *
         * @author Suriel Bendahan
         * @param id the item id to delete
         * @param cbOk callback to call on success with the result
         * @param cbError callback to call with an error object if the operation failed
         */
        this.remove = function(id, cbOk, cbError)
        {
            AjaxRequest.makeRequest(
            {
                contentType: 'application/json',
                dataType: 'json',
                type: 'DELETE',
                url: [baseUrl, userId, type, id, ''].join('/'),
                success: function(d)
                {
                    if (d.success) 
                    {
                        cbOk(d.result);
                    }
                    else 
                        if (cbError) 
                            cbError(d);
                },
                error: function(x, e)
                {
                    if (cbError) 
                        cbError(
                        {
                            success: false,
                            error: e
                        });
                }
            });
        };
        
        /**
         * Updates a bulk of items on the server
         *
         * @author Suriel Bendahan
         * @param items {Array} an array of items to update.
         * @param args {Object} a map of arguments to use as options. merge=MediaLibraryMergeTypes
         * @see MediaLibraryMergeTypes
         * @param cbOk callback to call on success with the result (a bulk result - Array containing the status of each item)
         * @param cbError callback to call with an error object if the operation failed
         */
        this.update = function(items, args, cbOk, cbError)
        {
            if (!_.isArray(items)) 
                items = [items];
				
            AjaxRequest.makeRequest(
            {
                contentType: 'application/json',
                data: items,
                dataType: 'json',
                type: 'POST',
                url: [baseUrl, userId, type, buildQString(args)].join('/'),
                success: function(d)
                {
                    if (d.success) 
                    {
                        _.each(d.result, function(idx, itm)
                        {
                            itm.item = items[idx];
                        });
                        cbOk(d.result);
                    }
                    else 
                        if (cbError) 
                            cbError(d);
                },
                error: function(x, e)
                {
                    if (cbError) 
                        cbError(
                        {
                            success: false,
                            error: e
                        });
                }
            });
        };
        
        /**
         * returns a list of items from the server
         *
         * @author Suriel Bendahan
         * @param args {Object} a map of arguments to use as options.
         * available options are skip:{Number},take:{Number},playlist:{String} (- to return items for a specific playlist)
         * @param cbOk callback to call on success with the result
         * @param cbError callback to call with an error object if the operation failed
         */
        this.getItems = function(args, cbOk, cbError)
        {
        
            AjaxRequest.makeRequest(
            {
                contentType: 'application/json',
                dataType: 'json',
                type: 'GET',
                url: [baseUrl, userId, type, 'items', buildQString(args)].join('/'),
                success: function(d)
                {
                    if (d.success) 
                    {
                        cbOk(d.result);
                    }
                    else 
                        if (cbError) 
                            cbError(d);
                },
                error: function(x, e)
                {
                    if (cbError) 
                        cbError(
                        {
                            success: false,
                            error: e
                        });
                }
            });
        };
        /**
         * returns the next set of result from the server
         *
         * @author Suriel Bendahan
         * @param paging {Object} the paging object returned from a previus call.
         * @param cbOk callback to call on success with the result
         * @param cbError callback to call with an error object if the operation failed
         */
        this.getNext = function(paging, cbOk, cbError)
        {
            if (typeof paging == 'string') 
                paging = 
                {
                    next: paging
                };
            if (!paging || !paging.next) 
                cbOk({}); //empty object???
            AjaxRequest.makeRequest(
            {
                contentType: 'application/json',
                dataType: 'json',
                type: 'GET',
                url: paging.next,
                success: function(d)
                {
                    if (d.success) 
                    {
                        cbOk(d.result);
                    }
                    else 
                        if (cbError) 
                            cbError(d);
                },
                error: function(x, e)
                {
                    if (cbError) 
                        cbError(
                        {
                            success: false,
                            error: e
                        });
                }
            });
        };
    };
    
    /**
     * Holds methods for manipulating media items for a user
     */
    this.Media = new itemManager('media');
    
    
    /**
     * Updates playlist info for a specifc media item
     *
     * @author Suriel Bendahan
     * @param itemId {String} the id of the media item to update
     * @param args {Object} a map of arguments to use as options.
     * available options are playlist:{String} - the playlist id the action refers to,
     * position:{Number} - the position in the playlist to add the item. leave empty (or null) to remove item from playlist
     * @param cbOk callback to call on success with the result
     * @param cbError callback to call with an error object if the operation failed
     */
    this.Media.setPlaylist = function(itemId, args, cbOk, cbError)
    {
        if (!args.playlist) 
            throw 'Playlist Id must be provided in args.playlist';
        
        AjaxRequest.makeRequest(
        {
            contentType: 'application/json',
            dataType: 'json',
            type: 'POST',
            url: [baseUrl, userId, 'media', itemId, 'playlist', buildQString(args)].join('/'),
            success: function(d)
            {
                if (d.success) 
                {
                    cbOk(d.result);
                }
                else 
                    if (cbError) 
                        cbError(d);
            },
            error: function(x, e)
            {
                if (cbError) 
                    cbError(
                    {
                        success: false,
                        error: e
                    });
            }
        });
    };
    
    
    /**
     * Holds methods for manipulating playlist objects for a user
     */
    this.Playlists = new itemManager('playlists');
    
    /**
     * Returns a library structure for the user
     *
     * @author Suriel Bendahan
     * @param args {Object} a map of arguments to use as options, available options are include:
     * 			{MediaLibraryIncludeItems} - the types of items to include
     * 		 	maxItems:{Number} - the max number of items to return for each playlist
     * @param cbOk callback to call on success with the result
     * @param cbError callback to call with an error object if the operation failed
     */
    this.getLibrary = function(args, cbOk, cbError)
    {
        AjaxRequest.makeRequest(
        {
            contentType: 'application/json',
            dataType: 'json',
            type: 'GET',
            url: [baseUrl, userId, 'library', buildQString(args)].join('/'),
            success: function(d)
            {
                if (d.success) 
                {
                    cbOk(d.result);
                }
                else 
                    if (cbError) 
                        cbError(d);
            },
            error: function(x, e)
            {
                if (cbError) 
                    cbError(
                    {
                        success: false,
                        error: e
                    });
            }
        });
    };
};




//use STORAGE_MEDIA_USER from service map
//var library = new MediaLibrary('722916258', 'http://storage.mobile.site-services.com/media');



