/**
 * This class is responsbile for storing and collecting usages,
 * and dumping them into the server
 *
 * @author Eran Zinman
 */
var Usages = (function()
{
	var me = {};
	
	// Array of existing usages, always hold the most current state of usages
	var usages = [];
	
	// Are we in "sending" mode? This is a state where we have sent data to server,
	// and still didn't get a reply.
	var isSending = false;
	
	// Key for storing the usages content
	var APP_USAGES_KEY = 'app_usages'
	
	/**
	 * Initialize the usages module, should be called on application
	 * start-up before we start tracking usages
	 * 
	 * @author Eran Zinman
	 */
	me.init = function()
	{
	    // Load app usages from local storage
        var stringifyUsages = conduitApi.platform.storage.getItem(null, null, APP_USAGES_KEY, true);
        if (stringifyUsages) 
        {
            // Parse cached item
            usages = JSON.parse(stringifyUsages);
			if (!usages || !_.isArray(usages))
			{
				// Make sure usages is an array
				usages = [];
			}
        }
	};
	
	/**
	 * Generate the usages header object
	 * 
	 * @return Usages header object
	 * @author Eran Zinman
	 */
	function generateUsagesHeader()
	{
		// Get the bitwised device token 
		var deviceType = DeviceToken.getDeviceToken();
						
		// Create the data structure
		var usagesHeader = 
		{
			'appId': APP_ID,
			'appVersion': APP_VERSION,
			'deviceType': deviceType,
			'latitude': location.coords.latitude,
			'longitude': location.coords.longitude,
			'timeStamp': Utils.TimeDate.utcTime(),
			'osVersion': 0,
			'platform': navigator.userAgent,
			'uuId': UserManager.getUserId()
		};
		
		return usagesHeader;
	}
		
	/**
	 * Send a bulk of events from the client storage to the server
	 * 
	 * @author Eran Zinman
	 */
	function sendEvents()
	{		
		// Get current usage count, if there are no usages, don't send
		if (usages.length == 0)
		{
			return;
		}
			
		// If we are in "sending" state, don't send until we get a reply
		if (isSending)
		{
			return;
		}
		
		// Flag that we are currently in "sending" state
		isSending = true;		

		// Generate usages header
		var usagesHeader = generateUsagesHeader();
		
		// Copy into variable the usages we are about to send
		var sentUsages = _.extend({}, usages);
				
		// Send usages
		var options = 
		{
			'postParams': 
			{
				'usageHeader': usagesHeader,
				'usageItems': usages
			}
		};
		
		// On successful usages send
		function onUsagesSentSuccess()
		{	
			// Remove already sent items from usages
			
			dumpUsagesToStorage();
			
			// We are done sending the usages
			isSending = false;	
		}
		
		// On failure usages send
		function onUsagesSentFailure()
		{
			// We are done sending
			isSending = false;	
		}			
		
		// Exceute service - post usages into server
		Services.executeService(UMS_SERVICE_NAME, null, onUsagesSentSuccess, onUsagesSentFailure, options);
	}	
	
	/**
     * Add new event to the usage stack
     *
     * @param eventType - type of the event
     * @param eventMeta - (optional) An object describing event meta data
     * @param triggerSend - boolean telling the usages manager to send the events now or not.
     * 
     * @author Eran Zinman
     */
    me.addEvent = function(eventType, eventMeta, triggerSend)
	{
		// Usages are disabled in simulator mode
		if (PLATFORM === platformEnum.simulator)
		{
			return;
		}

		// Create the event info structure		
		var eventInfo = 
		{
			timestamp: Utils.TimeDate.utcTime(),
			eventType: eventType,
			meta: eventMeta || {}
		};
		
		// Add event to pool
		usages.push(eventInfo);
		
		// Dump event pool to storage
		dumpUsagesToStorage();

		// Check if usage event must be sent now, or if we've passed the threshold, 
		// if we did - send events to the server
		if (triggerSend)
		{
			// Send events to server
			sendEvents();
		}
	};
		
	/**
     * Dump usages into local storage
     *
     * @author Eran Zinman
     */
    function dumpUsagesToStorage()
    {        
        // Write usages back to local storage
        var stringifyUsages = JSON.stringify(usages);
        
        // Save item into local storage
        conduitApi.platform.storage.setItem(null, null, APP_USAGES_KEY, stringifyUsages, true);
    }
	
	return me;
}());
